function [nmc, Kmean, Ksum, Kmax] = balanceExcThresh(nmc, p, Kfnc)

  if nargin < 2, p=1; end
  if nargin < 3, Kfnc={ @unirnd, 10, 20 }; end
  
  global_definitions

  pool = get(nmc, 'pool');
  postIdx  = [pool(p).neuronIdx];
  nIdx = length(postIdx);
  
  switch pool(p).type
   case 'LifNeuron'
    Apeak = inline('tau_s./(tau_m-tau_s).*((tau_s./tau_m).^(tau_s./(tau_m-tau_s))-(tau_s./tau_m).^(tau_m./(tau_m-tau_s)))');
   otherwise
    fprintf('Can not handle %s neurons! Command ignored.',pool(p).type);
    return;
  end

  if iscell(Kfnc)
    K=feval(Kfnc{:},1,nIdx);
  elseif isnumeric(Kfnc)
    K=Kfnc*ones(1,nIdx);
  end

  for i=1:nIdx
    
    [inSyn,outSyn]=csim('get',postIdx(i),'connections');

    Vthresh = csim('get',postIdx(i),'Vthresh');
    Rm = csim('get',postIdx(i),'Rm');
    Cm = csim('get',postIdx(i),'Cm');
    tau_m = Rm.*Cm;
    
    % find presynaptic neurons
    preIdx = zeros(1,length(inSyn));
    for s=1:length(inSyn)
      [preIdx(s),dummy]=csim('get',inSyn(s),'connections');
      preType(s)=csim('get',preIdx(s),'type');
      w(s)=csim('get',inSyn(s),'W');
      tau_s(s)=csim('get',inSyn(s),'tau');
    end
    
    % get type of neurons
    inhIdx = find(preType==INH);
    excIdx = find(preType==EXC);
    
    % calculate the peak of the EPSP of one Synapse
    A=Rm*w.*Apeak(tau_m,tau_s);

    % average EPSP amplitued
    meanEPSP=mean(A(excIdx));    
    sumEPSP=sum(A(excIdx));
    maxEPSP=max(A(excIdx));
    k = K(i)/meanEPSP*Vthresh;

    % scale w such that the average EPSP amplitude is (1/k)-th of Vthresh
    w(excIdx) = w(excIdx)*k;
    for s=1:length(inSyn)
      csim('set',inSyn(s),'W',w(s));
    end

    sumEPSP = k*sumEPSP;
    meanEPSP = k*meanEPSP;
    maxEPSP = k*maxEPSP;

    if ( sumEPSP < Vthresh )  
      fprintf('* % 3i: meanEPSP/Vthresh = %5.3g/%5.3g = %5.3g, sumEPSP/Vthresh = %5.3g/%5.3g = %5.3g\n',...
              i,meanEPSP,Vthresh,meanEPSP/Vthresh,sumEPSP,Vthresh,sumEPSP/Vthresh);
    end
    
    Kmean(i) = meanEPSP/Vthresh;
    Ksum(i) = sumEPSP/Vthresh;
    Kmax(i) = maxEPSP/Vthresh;
    
  end
  