/******************************************************************************
 *
 * $Id: defgen.cpp,v 1.4 2003/01/31 15:30:07 tnatschl Exp $
 *
 *
 * Copyright (C) 1997-2002 by Dimitri van Heesch.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation under the terms of the GNU General Public License is hereby 
 * granted. No representations are made about the suitability of this software 
 * for any purpose. It is provided "as is" without express or implied warranty.
 * See the GNU General Public License for more details.
 *
 * Documents produced by Doxygen are derivative works derived from the
 * input used in their production; they are not affected by this license.
 *
 */

#include <stdlib.h>

#include "qtbc.h"
#include "defgen.h"
#include "doxygen.h"
#include "message.h"
#include "config.h"
#include "classlist.h"
#include "util.h"
#include "defargs.h"
#include "outputgen.h"
#include "doc.h"
#include "dot.h"
#include "code.h"

#include <qdir.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qregexp.h>

#define DEF_DB(x)

//  inline void writeDEFString(QTextStream &t,const char *s)
//  {
//    const char* p=s;
//    char c;

//    t << '\'';
//    while ((c = *(p++)))
//    {
//      if (c == '\'')
//        t << '\\';
//      t << c;
//    }
//    t << '\'';
//  }


// #define max(A, B) ((A) > (B) ? (A) : (B))

//
// output the register line for a given class member
//
void generateDEFForMember(MemberDef *md, ClassDef *cd, QTextStream &t, QTextStream &lf)
{

  QCString memPrefix;

  if (md->memberType()==MemberDef::EnumValue) return;
  if (md->memberType()==MemberDef::Friend ) return;

  QCString typeStr = "";
  if (md->memberType()!=MemberDef::Define &&
      md->memberType()!=MemberDef::Enumeration
     )
  {
    typeStr = replaceAnonymousScopes(md->typeString());
  }

  QCString typeStrN = typeStr.simplifyWhiteSpace();

  //
  // we only consiter the following six types:
  //
  if ( typeStrN=="int" || typeStrN=="double" || typeStrN=="float" ||
       typeStrN=="int *" || typeStrN=="double *" || typeStrN=="float *" ||
       typeStrN=="int*" || typeStrN=="double*" || typeStrN=="float*" ) {

    //
    // default access is given by protection
    //
    int access=0;
    switch(md->protection()) {
    case Public:    access=2; /* readwrite */ break;
    case Protected: access=1; /* radonly */   break;
    case Private:   access=0; /* hidden */    break;
    }

    //
    // extrect the extra information part from the brief description
    //
    QCString brief=md->briefDescription();
    QRegExp r = QRegExp("\\[.*\\]");
    int i=brief.find(r);
    QCString reginfo=brief.mid(i,1000);
    reginfo=reginfo.simplifyWhiteSpace();
    brief = brief.left(i);
    brief = brief.stripWhiteSpace();
    brief = brief.replace( QRegExp("\""), "'" );

    //
    // if an explicit access method is given we overwrite the default
    //
    if ( reginfo.contains("readwrite") > 0 ) access = 2;
    if ( reginfo.contains("readonly") > 0 ) access = 1;
    if ( reginfo.contains("hidden") > 0 ) access = 0;
	   
    // 
    // for hidden variables nothing is written
    //
    if ( access < 1 ) return;

    //
    // access convert to string
    //
    QCString accessStr;
    switch ( access ) {
    case 1: accessStr = "READONLY";  break;
    case 2: accessStr = "READWRITE"; break;
    }

    //
    // look for units=<something>;
    //
    int j; QCString u="";
    if ( (i=reginfo.find(QRegExp("units\\s?=\\s?[^\\s;\\]]*\\s?;?\\]?"))) > -1 ) {
      u=reginfo.mid(i,1000);
      i=u.find('=');
      j=u.find(';'); if ( j < 0 ) j=u.find(']');
      u=u.mid(i+1,j-i-1);
      u=u.stripWhiteSpace();
    }
    QCString units=u;

    //
    // look for size=xyz;
    //
    u="1";
    if ( (i=reginfo.find(QRegExp("size\\s?=\\s?[^\\s;\\]]*\\s?;?\\]?"))) > -1 ) {
      if ( typeStrN.find('*') < 0 ) {
	QString cstr=cd->name();
	QString mstr=md->name();
	QString tstr=typeStrN;
	fprintf(stderr,"WARNING: Size info given for field '%s %s::%s'; reset to 1\n",
		tstr.ascii(),cstr.ascii(),mstr.ascii());
	u="1";
      } else {
	u=reginfo.mid(i,1000);
	i=u.find('=');
	j=u.find(';'); if ( j < 0 ) j=u.find(']');
	u=u.mid(i+1,j-i-1);
	u=u.stripWhiteSpace();
      }
    } else {
      if ( typeStrN.find('*') > -1 ) {
	QString cstr=cd->name();
	QString mstr=md->name();
	QString tstr=typeStrN;
	fprintf(stderr,"WARNING: No size info given for [array] field '%s%s::%s'\n",
		tstr.ascii(),cstr.ascii(),mstr.ascii());
	u="?";
      }
    }
    QCString size=u;

    //
    //  look for range=(lb,ub);
    //
    u=""; QCString lb="-1e30",ub="1e30";
    if ( (i=reginfo.find(QRegExp("range\\s?=\\s?\\([^\\s;,()\\]]+,[^\\s;,()\\]]+\\)\\s?;?\\]?"))) > -1 ) {
      u=reginfo.mid(i,1000);
      i=u.find('=');
      j=u.find(';'); if ( j < 0 ) j=u.find(']');
      u=u.mid(i+1,j-i-1);
      u=u.stripWhiteSpace();
      i=u.find('(');
      j=u.find(',');
      lb = u.mid(i+1,j-i-1);
      lb = lb.stripWhiteSpace();
      i=j;
      j=u.find(')');
      ub = u.mid(i+1,j-i-1);
      ub = ub.stripWhiteSpace();
    }

    //
    // output the proper line
    //
    t << "    REGFIELD(" 
      << "\"" << md->name() << "\"," 
      << md->name() << "," 
      << accessStr << ","
      << typeStrN  << ","
      << size << ","
      << lb << "," << ub << ","
      << "\"" << units << "\",\"" ;
    
    char *c=brief.data();
    if ( c!= 0 ) {
      while ( *c != 0 ) {
        switch ( *c ) {
        case '\\': t << "\\" ; break;
        }
	t << *(c++);
      }
    } else {
      t << "Missing description." ;
    }
    
    t << "\");" << endl;

    // \item[name] brief 
    //    lf << "\item[\texttt{" << md->name() << "}] "
    //   << brief << "\n"
    if ( units == "" ) 
      lf << "\\objfieldnu{" << accessStr << "}{";
    else 
      lf << "\\objfield{" << accessStr << "}{";

    const char *p=md->name();
    while ( *p != 0 ) {
      if ( *p == '_' ) lf << "\\" ;
      lf << *(p++) ;
    }
    lf << "}{" << lb << "}{" << ub << "}{" << units << "}{";

    c=brief.data();
    int insideEqu=0;
    if ( c!= 0 ) {
      while ( *c != 0 ) {
	if ( *c == '$' ) insideEqu = 1-insideEqu;
	if ( !insideEqu ) {
	  switch ( *c ) {
	  case '_': lf << "\\" ; break;
	  }
	}
	lf << *(c++);
      }
    }
    lf << "}\n" ;
  }
}


void generateDEFClassSection(ClassDef *cd,
    QTextStream &t,
    QTextStream &lf,
    MemberList *ml)
{
  //
  // loop over all members a certain list
  //
  if (ml->count()>0) {
    MemberListIterator mli(*ml);
    MemberDef *md;
    for (mli.toFirst();(md=mli.current());++mli)
    {
      generateDEFForMember(md,cd,t,lf);
    }
  }
}

void generateDEFForClass(QCString &outputDirectory, ClassDef *cd,QTextStream &t)
{

  if (cd->isReference()) return; // skip external references.
  if (cd->name().find('@')!=-1) return; // skip anonymous compounds.
  if (cd->templateMaster()!=0) return; // skip generated template instances.

  QCString fileName=outputDirectory+"/fields-"+cd->name()+".tex";
  QFile f(fileName);
  if (!f.open(IO_WriteOnly))
  {
    err("Cannot open file %s for writing!\n",fileName.data());
    return;
  }
  QTextStream lf(&f);

  //
  // output header for the class
  //
  QCString brief = cd->briefDescription();
  brief = brief.replace( QRegExp("\""), "'" );
  brief = brief.simplifyWhiteSpace();
  brief = brief.stripWhiteSpace();

  t << "  if (!" <<  cd->name() << "::classInfo ) {\n    " << cd->name() << "::classInfo =registerCsimClass(\"" << cd->name() << "\",\"" <<  brief << "\");\n";

  t <<  "    " << cd->name() << "*TmpObj = new " <<  cd->name() << ";" << endl;

  //
  //  output the REGFIELD macro
  //
  t << "    #define REGFIELD(_n,_f,_a,_t,_s,_l,_u,_un,_d) "
    << cd->name() << "::classInfo->registerField((char *)(TmpObj),_n,&(TmpObj->_f),_a,_s,_l,_u,_un,_d);"
    << endl;

  //
  // look for private, protected and public member ariables
  //
  int numMembers = 
    cd->pubAttribs.count()+ cd->proAttribs.count()+
    cd->priAttribs.count(); 
  if (numMembers>0)
  {
    generateDEFClassSection(cd,t,lf,&cd->pubAttribs);
    generateDEFClassSection(cd,t,lf,&cd->proAttribs);
    generateDEFClassSection(cd,t,lf,&cd->priAttribs);
  }

  //
  // undef the REGFIELD macro
  //
  t << "    #undef REGFIELD" << endl;
  t << "    delete TmpObj;" << endl;
  t << "  }" << endl << endl;
}

//
// Returns true if we should produce a register call for class cd. 
//
// We do so if it has a private static variable named classInfo except
// for the base class csimClass
// 
bool shouldMakeRegInfo(ClassDef *cd)
{
  if ( cd->name() == "csimClass" ) return 0;
  if ( cd->priStaticAttribs.count() > 0 ) {
    //
    // loop over all private static variables
    //
    MemberListIterator mli(cd->priStaticAttribs);
    MemberDef *md;
    for (mli.toFirst();(md=mli.current());++mli) {
      if ( md->name()=="classInfo" ) return TRUE;
    }
  }
  return FALSE;
}

void generateDEF()
{
  QCString outputDirectory = Config_getString("OUTPUT_DIRECTORY");
  if (outputDirectory.isEmpty())
  {
    outputDirectory=QDir::currentDirPath();
  }
  else
  {
    QDir dir(outputDirectory);
    if (!dir.exists())
    {
      dir.setPath(QDir::currentDirPath());
      if (!dir.mkdir(outputDirectory))
      {
        err("Error: tag OUTPUT_DIRECTORY: Output directory `%s' does not "
            "exist and cannot be created\n",outputDirectory.data());
        exit(1);
      }
      else if (!Config_getBool("QUIET"))
      {
        err("Notice: Output directory `%s' does not exist. "
            "I have created it for you.\n", outputDirectory.data());
      }
      dir.cd(outputDirectory);
    }
    outputDirectory=dir.absPath();
  }

  QDir dir(outputDirectory);
  if (!dir.exists())
  {
    dir.setPath(QDir::currentDirPath());
    if (!dir.mkdir(outputDirectory))
    {
      err("Cannot create directory %s\n",outputDirectory.data());
      return;
    }
  }
  QDir defDir(outputDirectory+"/def");
  if (!defDir.exists() && !defDir.mkdir(outputDirectory+"/def"))
  {
    err("Could not create def directory in %s\n",outputDirectory.data());
    return;
  }

  //
  // we generate the three files classlist.i, switch.i and registerclasses.i
  //
  QCString clfn=outputDirectory+"/classlist.i";
  QFile clf(clfn);
  if (!clf.open(IO_WriteOnly))
  {
    err("Cannot open file %s for writing!\n",clfn.data());
    return;
  }
  QTextStream clt(&clf);

  QCString sfn=outputDirectory+"/switch.i";
  QFile sf(sfn);
  if (!sf.open(IO_WriteOnly))
  {
    err("Cannot open file %s for writing!\n",sfn.data());
    return;
  }
  QTextStream st(&sf);

  QCString fileName=outputDirectory+"/registerclasses.i";
  QFile f(fileName);
  if (!f.open(IO_WriteOnly))
  {
    err("Cannot open file %s for writing!\n",fileName.data());
    return;
  }
  QTextStream t(&f);
 

  //
  // first we loop over all classes an write classlist.i, switch.i and the definitions
  // of the static classInfo variables for each class (into registerclasses.i)
  //
  if (Doxygen::classSDict.count()+Doxygen::inputNameList.count()>0)
  {
    ClassSDict::Iterator cli(Doxygen::classSDict);
    ClassDef *cd;
    int notfirst=0;
    for (cli.toFirst();(cd=cli.current());++cli) {
      if ( shouldMakeRegInfo(cd) ) {
	
	//
	// output the switch
	// 
	if ( notfirst++ ) st << " else " ;
	st << "if ( 0 == strcmp(className,\"" << cd->name() << "\") ) {" << endl;
	st << "#define TYPE " << cd->name() << endl;
	st << "__SWITCH_COMMAND__" << endl;
	st << "#undef TYPE" << endl;
	st << "}" ;


	//
	// output to a file with includes for each class
	//
	QCString fn=cd->getDefFileName();
	int i = fn.findRev('/');
	if ( i > -1 )
	  fn = fn.mid(i+1,2000);
	clt << "#include \"" << fn << "\"" << endl;

	//
	// outout static variable definitions
	//
	t << "csimClassInfo *" << cd->name() << "::classInfo=0;\n" ;
      }
    }
    //
    // finalize switch.i
    //
    st << " else switchError = -1;" << endl;
  }


  //
  // now we write the register function
  //
  t << endl << endl << "void csimClassInfoDB::registerClasses(void)" << endl << "{" << endl ;

  if (Doxygen::classSDict.count()+Doxygen::inputNameList.count()>0) {
    //
    // for each class write for each variable a suitable register call into registerfunctions.i
    //
    ClassSDict::Iterator cli(Doxygen::classSDict);
    ClassDef *cd;
    for (cli.toFirst();(cd=cli.current());++cli) {
      if ( shouldMakeRegInfo(cd) )
	generateDEFForClass(outputDirectory,cd,t);
    }
  }

  //
  // finilize the register function
  //
  t << endl << "}" << endl;
}
