#include "mex.h"
#include "math.h"

/*
**
** Here comes the MATLAB INTERFACE: 
**
** X=SPIKES2PULSE(C,t,tau1,tau2)
**
*/

/* Input Arguments */

#define	C_IN	 prhs[0]
#define	t_IN	 prhs[1]
#define	tau1_IN   prhs[2]
#define	tau2_IN   	 prhs[3]

/* Output Arguments */

#define X_OUT  plhs[0]

/*
** some helpers do parse the MatLab given input
*/
#if !defined(max)
#define	max(A, B)	((A) > (B) ? (A) : (B))
#endif
#if !defined(min)
#define	min(A, B)	((A) < (B) ? (A) : (B))
#endif

int getDouble(const mxArray *arg, double *x)
{
  int m = mxGetM(arg); 
  int n = mxGetN(arg); 
  if (!mxIsNumeric(arg) || mxIsComplex(arg) || 
      mxIsSparse(arg)  || !mxIsDouble(arg) || 
      (m != 1) || (n != 1)) {
    *x = 0.0;
    return -1;
  }
  *x = mxGetScalar(arg);
  return 0;
}

int getDoubleVector(const mxArray *arg, double **x, int *n)
{
  int m = min(mxGetM(arg),mxGetN(arg));
  *n = max(mxGetM(arg),mxGetN(arg));
  if (!mxIsNumeric(arg) ||  mxIsComplex(arg) ||  
       mxIsSparse(arg)  || !mxIsDouble(arg)  || m > 1 || m < 0) {
    *x = NULL;
    return -1;
  }
  *x = mxGetPr(arg); 
  return 0;
}

/*
** the mexFunction the the gateway to and from matlab
*/  
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  int	m,n,nSTP,nDIM,i_data=0,nSpikes,c,i,j;
  double *t,tau1,tau2,*st,*x,dt,tmp;
  mxArray *st_a;

  /* 
  ** check arguments 
  */
  if (nrhs < 2)
    mexErrMsgTxt("SPIKES2PULSE requires at least 2 input arguments!\n");

  if (nlhs != 1) 
    mexErrMsgTxt("SPIKES2PULSE requires 1 output argument!\n");

  /* C should be something like a struct or an empty matrix */
  n = mxGetN(C_IN);
  m = mxGetM(C_IN);
  nDIM = max(n,m);

  if ( nDIM > 0 && !mxIsStruct(C_IN) )
    mexErrMsgTxt("SPIKES2PULSE requires that C is a struct\n");

  if ( nDIM == 0 )
    mexErrMsgTxt("SPIKES2PULSE: nothing to do since there are 0 channels!\n");

  if ( nDIM> 0 && min(m,n) != 1 )
    mexErrMsgTxt("SPIKES2PULSE: C is not a struct vector.\n");
  
  if ( nDIM > 0 ) {
    i_data = mxGetFieldNumber(C_IN,"data");
    if ( i_data < 0 )
      mexErrMsgTxt("C has no field 'data' \n");
  }
    
  /*
  ** check t
  */
  if ( getDoubleVector(t_IN,&t,&nSTP) < 0 )
    mexErrMsgTxt("SPIKES2PULSE requires that 't' is a sortet vector!\n");

  tau1=30e-1;
  if ( nrhs > 2 ) {
    if ( getDouble(tau1_IN,&tau1) < 0 )
      mexErrMsgTxt("SPIKES2PULSE requires that 'tau1' is a double scalar!\n");
    if ( tau1 <=0 ) mexErrMsgTxt("SPIKES2PULSE requires that tau1 > 0!");
  }

  /*
  ** check tau2
  */
  tau2=31e-3;
  if ( nrhs > 3 ) {
    if ( getDouble(tau2_IN,&tau2) < 0 )
      mexErrMsgTxt("SPIKES2PULSE requires that 'tau2' is a double scalar!\n");
    if ( tau2 <=0 ) mexErrMsgTxt("SPIKES2PULSE requires that tau2 > 0!");
  }

  /* create outputmatrix */
  X_OUT = mxCreateDoubleMatrix( nSTP, nDIM, mxREAL);
  x=mxGetPr(X_OUT);

  for(c=0;c<nDIM;c++) {
    st_a = mxGetFieldByNumber(C_IN,c,i_data);
    st=mxGetPr(st_a);
    nSpikes=mxGetN(st_a);
    /*   printf("%i spikes\n",nSpikes); */
    for(j=0;j<nSTP;j++) {
      *x=0.0;
      for(i=0; ( i<nSpikes ) && (st[i]<=t[j]); i++) {
	dt=st[i]-t[j];
	__PULSE_TYPE__
	/* *x += exp(dt/tau1); */
	/* *x += (exp(dt/tau1)-exp(dt/tau2)); */
	/* *x += (int)(dt>-tau1); */
      }
      x++;
    }
  }
}
