function [train_states,S1,S2,nN23f,nN5f,IdxE,IdxI] = collect_all_data_2input_2output_l25(nmc,binwidth,varargin)

global VERBOSE_LEVEL

if nargin == 2
   error('Input type not implemented!')
elseif nargin > 2
   S1in = varargin{1};
   S2in = varargin{2};
   nS = varargin{3};
   if nargin > 3
      Tstim = varargin{4};
   else
      Tstim = max([get(S1in,'Tstim') get(S2in,'Tstim')]);
   end
 
   for i=1:nS
      S1(i) = generate(S1in,Tstim ,i);
      S1(i).info.Tstim = Tstim;

      S2(i) = generate(S2in,Tstim ,i);
      S2(i).info.Tstim = Tstim;
   end

else
   error('Too many input arguments!')
end

% get readout neuron number for readout 4 & 5

if nargin > 6
   nN23f = varargin{5};
else
   nN23f = [];
end

if nargin > 7
   nN5f = varargin{6};
else
   nN5f = [];
end

if nargin > 9
   IdxE = varargin{7};
   IdxI = varargin{8};
   IdxDef = 1;
else
   IdxDef = 0;
end

nPools =  length(get(nmc.template,'pool'));

% transform input

clear S

for nS = 1:length(S2)

   S{nS} = S2(nS);
   nCh1 = length(S1(nS).channel);
   nCh2 = length(S2(nS).channel);

   chIdx1 = 1:nCh1;
   chIdx2 = 1:nCh2;
   chIdx = 0;

   % input to layer 2/3
   S{nS}.channel(chIdx2+chIdx) = S2(nS).channel;

   % input to layer 4 
   chIdx = chIdx + nCh2;
   S{nS}.channel(chIdx2+chIdx) = S2(nS).channel(:);

   % input to layer 5
   chIdx = chIdx + nCh2;
   S{nS}.channel(chIdx2+chIdx) = S2(nS).channel(:);

   % 2nd input channel to layer 2/3
   chIdx = chIdx + nCh2;
   S{nS}.channel(chIdx1+chIdx) = S1(nS).channel(:);
end

tic
R = simulate(pool_circuit_template,nmc,S{:});

if VERBOSE_LEVEL, toc, end

DISP = 1;
if DISP & (length(S)==1)
  plot_spike_raster(R,nmc);
end


% calculate readout (2/3E & 5E neurons)
%----------------------------------------

get_readoutidx

clear readoutIdxE readoutIdxI

if isempty(nN23f)||isempty(nN5f)
   % create readouts with same number of neurons as L23 and L5 readouts but 
   % neurons taken randomly chosen throughout the circuit

   nN = sum(nNall);
   idx4 = zeros(1,nN);
   idx5 = zeros(1,nN);

   i = randperm(nN);
   idx4(i(1:sum(nN23))) = 1;
   i = randperm(nN);
   idx5(i(1:sum(nN5))) = 1;

   for nP = 1:length(pool_size)
      i = find(idx4);
      i(i > sum(pool_size(1:nP))) = [];
      idx4(i) = 0;
      nN23f(nP) = length(i);
      i = find(idx5);
      i(i > sum(pool_size(1:nP))) = [];
      idx5(i) = 0;
      nN5f(nP) = length(i);
   end
end


for nP = 1:nPools

   if (IdxDef == 0)
      neuronIdx = get(nmc.circuit,'pool',{nP},'neuronIdx');
      neuronType = csim('get',neuronIdx,'type');

      IdxE{nP} = find(neuronType == 2);
      IdxI{nP} = find(neuronType == 1);
      IdxE{nP} = IdxE{nP}(randperm(length(IdxE{nP})));
      IdxI{nP} = IdxI{nP}(randperm(length(IdxI{nP})));
   end

   % readout from all neurons in the circuit

   readoutIdxE{nP,1} = IdxE{nP};
   readoutIdxI{nP,1} = IdxI{nP};

   % readout from L23 neurons in the circuit

   readoutIdxE{nP,2} = IdxE{nP}(1:nN23(nP*2-1));
   readoutIdxI{nP,2} = IdxI{nP}(1:nN23(nP*2));

   % readout from L5 neurons in the circuit

   readoutIdxE{nP,3} = IdxE{nP}(1:nN5(nP*2-1));
   readoutIdxI{nP,3} = IdxI{nP}(1:nN5(nP*2));

   % readout from same number of neurons as L23 readout in the circuit

   readoutIdxE{nP,4} = IdxE{nP}(1:nN23f(nP*2-1));
   readoutIdxI{nP,4} = IdxI{nP}(1:nN23f(nP*2));

   % readout from same number of neurons as L23 readout in the circuit

   readoutIdxE{nP,5} = IdxE{nP}(1:nN5f(nP*2-1));
   readoutIdxI{nP,5} = IdxI{nP}(1:nN5f(nP*2));
end


nRST = size(readoutIdxE,2);

for nR = 1:length(R)
   for i = 1:length(binwidth)
      for rst = 1:nRST
         train_states{rst}(nR).X{i} = [];
      end
   end

   for nP = 1:nPools

      for i = 1:length(binwidth)
         X = response2states(R{nR}.R{nP},[],'[0.001:0.001:Tsim]',{ 'spikes2exp' binwidth(i) }); 

         % store last liquid state
         for rst = 1:nRST
            train_states{rst}(nR).X{i} = [train_states{rst}(nR).X{i} +X.X(end,readoutIdxE{nP,rst})];
            train_states{rst}(nR).X{i} = [train_states{rst}(nR).X{i} -X.X(end,readoutIdxI{nP,rst})];
         end
      end

      % record layer statistic and save for the first readout

      train_states{1}(nR).layer(nP).ActNeurons = 0;
      T1 = 2/3*Tstim;

      firingrate =[];
      spikecount =[];
      CV = [];
      ActNeurons = 0;
      for nN = 1:length(R{nR}.R{nP}.channel)
         st = R{nR}.R{nP}.channel(nN).data(R{nR}.R{nP}.channel(nN).data > T1);
         ActNeurons = ActNeurons + ~isempty(st);
         firingrate = [firingrate st/( Tstim - T1)];
         spikecount = [spikecount length(st)];
         CV = [CV std(diff(st))/mean(diff(st))] ;
      end
      train_states{1}(nR).layer(nP).ActNeurons = ActNeurons;
      train_states{1}(nR).layer(nP).mfr = mean(firingrate);
      train_states{1}(nR).layer(nP).vfr = var(firingrate);
      train_states{1}(nR).layer(nP).msc = mean(spikecount);
      train_states{1}(nR).layer(nP).vsc = var(spikecount);
      train_states{1}(nR).layer(nP).mCV = mean(CV(~isnan(CV)));
      train_states{1}(nR).layer(nP).vCV = var(CV(~isnan(CV)));
      train_states{1}(nR).layer(nP).lCV = length(CV(~isnan(CV)));
      train_states{1}(nR).layer(nP).mrCV = mean(CV(~isnan(CV) & CV~=0));
      train_states{1}(nR).layer(nP).vrCV = var(CV(~isnan(CV) & CV~=0));
      train_states{1}(nR).layer(nP).lrCV = length(CV(~isnan(CV) & CV~=0));
   end

   for rst = 1:nRST
      train_states{rst}(nR).t = S{nR}.info.Tstim;
   end
end















