function [pct] = make_network_fanin_HH(Cscale,Wscale,WINscale,nInputs,layer_base,Wset,noise);
% conserves distribution of number of input and output synapses
% but mixes all neurons in the circuits, so that the input is fed in randomly to hubs


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% generate a POOL CIRCUIT TEMPLATE
%
% Neuron type: conductance based neurons and synapses
%              with background conductance 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (nargin ~= 7)
   error('Number of input arguments must be six.')
end


if length(WINscale)~=4
   error('WINscale must have 4 elements, one for each input.')
end


II = 1; IE = 2; EI = 3; EE = 4;


WscaleTotal = 12;

Wscale = Wscale * WscaleTotal;

WINscale = WINscale * [1 1 1 1];
WINscale = WINscale * WscaleTotal;


def = get(neural_microcircuit,'def'); % get default pool and connection parameters
pct =  pool_circuit_template;         % create empty pool circuit template



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% general simulation parameters
%

dt_sim = 2e-5;
pct = set(pct,'dt_sim',dt_sim);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% create recorders
%

recorder(1) = deal( get(pct,'recorder') );    % get recorder structure
recorder(1).Field = 'spikes';
pct = set(pct,'recorder',recorder);		% set modified recorders



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% create layers
%
%    neurons per layer distribution based on Ani's paper p. 4
%

% general layer parameters
%----------------------------------------

A = 34636e-12; % [m]

gl = 0.045e-3/1e-4*A; % [S]
Rm = 1/gl;
Cm = 1e-6/1e-4*A;

Vresting = -80e-3;         % Volt

nn_scaling = 25/prod(layer_base);

ge0 = 0.012e-6;   % parameters for Ornstein-Uhlenbeck noise
sig_e = 0.003e-6*(noise~=0);
tau_e = 2.7e-3;

gi0 = 0.057e-6;
sig_i = 0.0066e-6*(noise~=0);  
tau_i = 10.5e-3;

Ee = 0e-3;
Ei = -75e-3;

% create pools
%----------------------------------------

pool(1:3) = deal( get(pct,'pool') );		% get pool structure
[pool(:).parameters] = deal(def.pool); 		% set default pool parameters



% set layer 2/3 parameters
%----------------------------------------

pool(1).parameters.size = [layer_base 3];
pool(1).add.origin = [3 1 10];

[pool(1).parameters.Neuron.Inoise] = deal([0 0]); % no background current noise
[pool(1).parameters.Neuron.Iinject] = deal([0 0]);

pool(1).parameters.type = 'CbHHOuNeuron';
[pool(1).parameters.Neuron.Rm] = deal(Rm);
[pool(1).parameters.Neuron.Cm] = deal(Cm);
[pool(1).parameters.Neuron.Trefract] = deal(3e-3);
[pool(1).parameters.Neuron.Vthresh] = deal(-30e-3);
[pool(1).parameters.Neuron.Vresting] = deal([Vresting]);
[pool(1).parameters.Neuron.Ee] = deal([Ee]);
[pool(1).parameters.Neuron.Ei] = deal([Ei]);

[pool(1).parameters.Neuron.ge0] = deal(ge0);	% set background conductance noise
[pool(1).parameters.Neuron.sig_e] = deal(sig_e);
[pool(1).parameters.Neuron.tau_e] = deal(tau_e);

[pool(1).parameters.Neuron.gi0] = deal(gi0);
[pool(1).parameters.Neuron.sig_i] = deal(sig_i);
[pool(1).parameters.Neuron.tau_i] = deal(tau_i);



% set layer 4 parameters
%----------------------------------------

pool(2).parameters.size = [layer_base 2];
pool(2).add.origin = [3 1 7];

[pool(2).parameters.Neuron.Inoise] = deal([0 0]); % no background current noise
[pool(2).parameters.Neuron.Iinject] = deal([0 0]);

pool(2).parameters.type = 'CbHHOuNeuron';
[pool(2).parameters.Neuron.Rm] = deal(Rm);
[pool(2).parameters.Neuron.Cm] = deal(Cm);
[pool(2).parameters.Neuron.Trefract] = deal(3e-3);
[pool(2).parameters.Neuron.Vthresh] = deal(-30e-3);
[pool(2).parameters.Neuron.Vresting] = deal([Vresting]);
[pool(2).parameters.Neuron.Ee] = deal([Ee]);
[pool(2).parameters.Neuron.Ei] = deal([Ei]);

[pool(2).parameters.Neuron.ge0] = deal(ge0);	% set background conductance noise
[pool(2).parameters.Neuron.sig_e] = deal(sig_e);
[pool(2).parameters.Neuron.tau_e] = deal(tau_e);

[pool(2).parameters.Neuron.gi0] = deal(gi0);
[pool(2).parameters.Neuron.sig_i] = deal(sig_i);
[pool(2).parameters.Neuron.tau_i] = deal(tau_i);



% set layer 5 parameters
%----------------------------------------

pool(3).parameters.size = [layer_base 5];
pool(3).add.origin = [3 1 1];

[pool(3).parameters.Neuron.Inoise] = deal([0 0]); % no background current noise
[pool(3).parameters.Neuron.Iinject] = deal([0 0]);

pool(3).parameters.type = 'CbHHOuNeuron';
[pool(3).parameters.Neuron.Rm] = deal(Rm);
[pool(3).parameters.Neuron.Cm] = deal(Cm);
[pool(3).parameters.Neuron.Trefract] = deal(3e-3);
[pool(3).parameters.Neuron.Vthresh] = deal(-30e-3);
[pool(3).parameters.Neuron.Vresting] = deal([Vresting]);
[pool(3).parameters.Neuron.Ee] = deal([Ee]);
[pool(3).parameters.Neuron.Ei] = deal([Ei]);

[pool(3).parameters.Neuron.ge0] = deal(ge0);	% set background conductance noise
[pool(3).parameters.Neuron.sig_e] = deal(sig_e);
[pool(3).parameters.Neuron.tau_e] = deal(tau_e);

[pool(3).parameters.Neuron.gi0] = deal(gi0);
[pool(3).parameters.Neuron.sig_i] = deal(sig_i);
[pool(3).parameters.Neuron.tau_i] = deal(tau_i);


if (1)
 [pool(1).parameters.Neuron.Vinit] = deal([-70e-3 -60e-3]);
 [pool(2).parameters.Neuron.Vinit] = deal([-70e-3 -60e-3]);
 [pool(3).parameters.Neuron.Vinit] = deal([-70e-3 -60e-3]);
end



pct = set(pct,'pool',pool);			% set pool structure


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% create connections
%
%   Alex Thomson et al., Cer. Cortex 2002
%   Dantsker / Callaway, Nature 2000
%   Douglas, Martin, p.479, p.53?
%   

% general connection parameters
%----------------------------------------


nPools = length(pool) ;  

E = [Ei Ee Ei Ee];
V = -65e-3;            % mean voltage to calculate peak of the synaptic conductance

tau = [0.006 0.003 0.006 0.003];


[W,C] = set_connection_matrix(Rm,Wset{:});

W = abs(W);

Wscale = Wscale * nn_scaling; % normalize to number of synapses

% create connections
%----------------------------------------

conn(1:nPools,1:nPools) = deal( get(pct,'conn') );	% get connection structure
[conn(:).parameters] = deal(def.conn);	 		% set default connection parameters

Volume = 0;
for nPool = 1:length(pool)
   Volume = Volume + prod(pool(nPool).parameters.size);
end

% calculate neuron indices

idxC = [1 0 1 0 1 0];
Csyntype = repmat(idxC',[1 nPools*2])*2 + repmat(idxC,[nPools*2 1]) + 1;
Csyntype = Csyntype'; % because pre and post notation is flipped for the C matrix

nNeurons = [];
for nPool = 1:length(pool)
   nNeurons(nPool) = prod(pool(nPool).parameters.size);
end

nNeurons = [ceil(0.8*nNeurons); floor(0.2*nNeurons)];
layeridx = nNeurons;
nNeurons = repmat(nNeurons(:),[1 6]);
nNeurons = nNeurons'.*nNeurons;

i = [0 cumsum(layeridx(:)')];


% keep order so that first 80% of indices in each layer are excitatory
for j = 1:length(i)-1
   NLidx{j} = [i(j):i(j+1)-1];
end


for nPre = 1:nPools
   for nPost = 1:nPools

      Cs = C(2*nPre - [1 0],2*nPost - [1 0]); % get Cs from C matrix in order [II EI IE EE]
      Cs = fliplr(Cs(:)');
      Ws = W(2*nPre - [1 0],2*nPost - [1 0]); % get Ws from W matrix in order [II EI IE EE]
      Ws = fliplr(Ws(:)');

      conn(nPost,nPre).parameters.type = 'DynamicSpikingCbSynapse';
      conn(nPost,nPre).parameters.lambda = Inf;
      conn(nPost,nPre).parameters.C = Cs;
      conn(nPost,nPre).parameters.Wscale = Wscale;
      conn(nPost,nPre).parameters.Cscale = Cscale;

      % calculate number of synapses and select neuron indices for this connection

      conn(nPost,nPre).add.DefIdxConn = [];

      for preIdx = 2*nPre - [1 0]
         for postIdx = 2*nPost - [1 0]
            conntype = Csyntype(preIdx,postIdx);

            % take corresponding fraction of already generated synapses
            synnumb = round(nNeurons(preIdx,postIdx).*C(preIdx,postIdx));

            % make sure that not the same pre, post neuron pair is chosen several times
            if (length(NLidx{postIdx})*length(NLidx{preIdx})) < synnumb
               error('Too many number of synapses for this connection.')
            end

            idx = randperm(length(NLidx{postIdx})*length(NLidx{preIdx}));
            prei = mod(idx-1,length(NLidx{preIdx}))+1;
            posti = floor((idx-1)/length(NLidx{preIdx}))+1;

            conn(nPost,nPre).add.dest{conntype} = NLidx{postIdx}(posti(1:synnumb))'; 
            conn(nPost,nPre).add.src{conntype} = NLidx{preIdx}(prei(1:synnumb))';  
         end
      end 

      for nS = 1:4
         conn(nPost,nPre).parameters.Synapse(nS).tau = tau(nS); % [sec]
         conn(nPost,nPre).parameters.Synapse(nS).E = E(nS); % [V]
         conn(nPost,nPre).parameters.Synapse(nS).W = Ws(nS)/abs(E(nS)-V);
      end
   end
end

% randomize network connections by preneuron index exchange
%----------------------------------------------------------

%%%%%%%%%%%%%%%%%%%
% read index values

idx = {[],[],[],[]};
idx2 = {[],[],[],[]};
lidx = [0,0,0,0];
for nPre = 1:nPools
   for nPost = 1:nPools
      for conntype = [II IE EI EE]
         l = length(conn(nPost,nPre).add.src{conntype});
         idx{conntype}(lidx(conntype)+1:lidx(conntype)+l) =  ...
            conn(nPost,nPre).add.src{conntype};
         idx2{conntype}(lidx(conntype)+1:lidx(conntype)+l) =  ...
            conn(nPost,nPre).add.dest{conntype};
         lidx(conntype) = lidx(conntype) + l;
      end
   end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% change neuron indices to scramble input to hubs structure


disp('Switch neuron indices.')

nidxE = [NLidx{1:2:end}]; % get indices of exitatory neurons
nidxI = [NLidx{2:2:end}]; % get indices of inhibitory neurons

% scramble exc and inh neuron indices
nidx = [];
nidx(nidxE+1) = nidxE(randperm(length(nidxE)));  % +1 because lowest index is 0
nidx(nidxI+1) = nidxI(randperm(length(nidxI)));

% switch pre and post synatpic neuron indices to a different neuron of the same type
for conntype = [II IE EI EE]
   idx{conntype} = nidx(idx{conntype}+1);
   idx2{conntype} = nidx(idx2{conntype}+1);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% exchange pre neuron index of synapses of same type

CM = zeros(Volume);

% generate connectivity matrix (here because conntypes cannot interfere)
for conntype = [II IE EI EE]
   i = sub2ind(size(CM),idx2{conntype}+1,idx{conntype}+1);
   CM(i) = 1;
end

disp('Switch pre synaptic neuron indices.')
for conntype = [II IE EI EE]
   for nsw = 1:(3*lidx(conntype))
      % choose two synapses
      i = ceil(rand(1,2)*lidx(conntype)); % randperm would be too slow
      i2 = i([2 1]);

      % switch src neuron index if synapses don't exist
      if (CM(idx2{conntype}(i(1))+1,idx{conntype}(i2(1))+1)==0)&...
         (CM(idx2{conntype}(i(2))+1,idx{conntype}(i2(2))+1)==0)

         CM(idx2{conntype}(i(1))+1,idx{conntype}(i(1))+1) = 0;
         CM(idx2{conntype}(i(2))+1,idx{conntype}(i(2))+1) = 0;

         CM(idx2{conntype}(i(1))+1,idx{conntype}(i2(1))+1) = 1;
         CM(idx2{conntype}(i(2))+1,idx{conntype}(i2(2))+1) = 1;

         idx{conntype}(i2) = idx{conntype}(i);

      end 
   end
end

%%%%%%%%%%%%%%%%%%%
% write index values

lidx = [0,0,0,0];
for nPre = 1:nPools
   for nPost = 1:nPools
      for conntype = [II IE EI EE]
         l = length(conn(nPost,nPre).add.src{conntype});
         conn(nPost,nPre).add.src{conntype} = ...
             idx{conntype}(lidx(conntype)+1:lidx(conntype)+l)';
         conn(nPost,nPre).add.dest{conntype} = ...
            idx2{conntype}(lidx(conntype)+1:lidx(conntype)+l)';
         lidx(conntype) = lidx(conntype) + l;
      end
   end
end

pct = set(pct,'conn',conn(:));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% create inputs
%


nInChannels = 4;
input(1:nInChannels) = deal(get(pct,'input'));
[input(:).parameters] = deal(def.conn);

Winscale = WINscale / nInputs;


fprintf('Final Winscale: %g,%g,%g,%g\n',Winscale')
pause(1)


% input to layer 2/3
%----------------------------------------

input(1).parameters.type = 'StaticSpikingCbSynapse';
input(1).parameters.lambda = Inf;
input(1).parameters.C = [0.0 0.0 0.0 0.2];
input(1).parameters.Wscale = Winscale(1);

input(1).add.randConn = [];
[input(1).parameters.Synapse(1:4).E] = deal(Ee); % [V]
input(1).parameters.rescale = 0;
for nS = 1:4, input(1).parameters.Synapse(nS).W = input(1).parameters.Synapse(EE).W/abs(E(nS)-V); end


% input to layer 4
%----------------------------------------

input(2).parameters.type = 'StaticSpikingCbSynapse';
input(2).parameters.lambda = Inf;
input(2).parameters.C = [0.0 0.5 0.0 0.8];

input(2).parameters.Wscale = Winscale(2);

pause(0.5)

input(2).add.randConn = [];

[input(2).parameters.Synapse(1:4).E] = deal(Ee); % [V]
input(2).parameters.rescale = 0;
for nS = 1:4, input(2).parameters.Synapse(nS).W = input(2).parameters.Synapse(EE).W/abs(E(nS)-V); end

% input to layer 5
%----------------------------------------

input(3).parameters.type = 'StaticSpikingCbSynapse';
input(3).parameters.lambda = Inf;
input(3).parameters.C = [0.0 0.0 0.0 0.1];
input(3).parameters.Wscale = Winscale(3);

input(3).add.randConn = [];
[input(3).parameters.Synapse(1:4).E] = deal(Ee); % [V]
input(3).parameters.rescale = 0;
for nS = 1:4, input(3).parameters.Synapse(nS).W = input(3).parameters.Synapse(EE).W/abs(E(nS)-V); end


% 2nd input channel to layer 2/3
%----------------------------------------

input(4).parameters.type = 'StaticSpikingCbSynapse';
input(4).parameters.lambda = Inf;
input(4).parameters.C = [0.0 0.0 0.0 0.2];
input(4).parameters.Wscale = Winscale(4);
input(4).add.randConn = [];
[input(4).parameters.Synapse(1:4).E] = deal(Ee); % [V]
input(4).parameters.rescale = 0;
for nS = 1:4, input(4).parameters.Synapse(nS).W = input(4).parameters.Synapse(EE).W/abs(E(nS)-V); end

Iidx = repmat([1:nInChannels]',[1 nInputs])';
Iidx = Iidx(:);

input(1:nInputs*nInChannels) = input(Iidx);

pct = set(pct,'input',input);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% set input and output channels
%

INidx = mod(Iidx-1,nPools)+1;
pct = set(pct,'INidx',INidx);
pct = set(pct,'OUTidx',[1:nPools]);


